@@ -27,27 +27,36 @@
 #include <string.h>
 #include <gconf/gconf-client.h>
 #include "vino-util.h"
+#include "vino-mdns.h"
 
 #define VINO_PREFS_DIR                    "/desktop/gnome/remote_access"
 #define VINO_PREFS_ENABLED                VINO_PREFS_DIR "/enabled"
 #define VINO_PREFS_PROMPT_ENABLED         VINO_PREFS_DIR "/prompt_enabled"
 #define VINO_PREFS_VIEW_ONLY              VINO_PREFS_DIR "/view_only"
+#define VINO_PREFS_LOCAL_ONLY             VINO_PREFS_DIR "/local_only"
+#define VINO_PREFS_USE_ALTERNATIVE_PORT   VINO_PREFS_DIR "/use_alternative_port"
+#define VINO_PREFS_ALTERNATIVE_PORT       VINO_PREFS_DIR "/alternative_port"
 #define VINO_PREFS_REQUIRE_ENCRYPTION     VINO_PREFS_DIR "/require_encryption"
 #define VINO_PREFS_AUTHENTICATION_METHODS VINO_PREFS_DIR "/authentication_methods"
 #define VINO_PREFS_VNC_PASSWORD           VINO_PREFS_DIR "/vnc_password"
+#define VINO_PREFS_LOCK_SCREEN            VINO_PREFS_DIR "/lock_screen_on_disconnect"
 
-#define VINO_N_LISTENERS 6
+#define VINO_N_LISTENERS 10
 
 static GConfClient *vino_client  = NULL;
 static GSList      *vino_servers = NULL;
 static guint        vino_listeners [VINO_N_LISTENERS] = { 0, };
 
-static gboolean        vino_enabled            = FALSE;
-static gboolean        vino_prompt_enabled     = FALSE;
-static gboolean        vino_view_only          = FALSE;
-static gboolean        vino_require_encryption = FALSE;
-static VinoAuthMethod  vino_auth_methods       = VINO_AUTH_VNC;
-static char           *vino_vnc_password       = NULL;
+static gboolean        vino_enabled              = FALSE;
+static gboolean        vino_prompt_enabled       = FALSE;
+static gboolean        vino_view_only            = FALSE;
+static gboolean        vino_local_only           = FALSE;
+static gboolean        vino_require_encryption   = FALSE;
+static VinoAuthMethod  vino_auth_methods         = VINO_AUTH_VNC;
+static char           *vino_vnc_password         = NULL;
+static gboolean        vino_use_alternative_port = FALSE;
+static int             vino_alternative_port     = VINO_SERVER_DEFAULT_PORT;
+static gboolean        vino_lock_screen          = FALSE;
 
 
 static void
@@ -70,6 +79,11 @@ vino_prefs_enabled_changed (GConfClient *client,
   
   dprintf (PREFS, "Access enabled changed: %s\n", vino_enabled ? "(true)" : "(false)");
 
+  if (vino_enabled)
+    vino_mdns_start ();
+  else
+    vino_mdns_stop ();
+
   for (l = vino_servers; l; l = l->next)
     vino_server_set_on_hold (l->data, !enabled);
 }
@@ -122,6 +136,30 @@ vino_prefs_view_only_changed (GConfClient *client,
     vino_server_set_view_only (l->data, view_only);
 }
 
+static void
+vino_prefs_local_only_changed (GConfClient *client,
+			       guint	    cnxn_id,
+			       GConfEntry  *entry)
+{
+  gboolean  local_only;
+  GSList   *l;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  local_only = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (vino_local_only == local_only)
+    return;
+
+  vino_local_only = local_only;
+
+  dprintf (PREFS, "Local only changed: %s\n", vino_local_only ? "(true)" : "(false)");
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_local_only (l->data, local_only);
+}
+
 static void
 vino_prefs_require_encryption_changed (GConfClient *client,
 				       guint        cnxn_id,
@@ -233,22 +271,101 @@ vino_prefs_vnc_password_changed (GConfClient *client,
     vino_server_set_vnc_password (l->data, vnc_password);
 }
 
+static void
+vino_prefs_use_alternative_port_changed (GConfClient *client,
+                                         guint        cnxn_id,
+                                         GConfEntry  *entry)
+{
+  gboolean  enabled;
+  GSList   *l;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  enabled = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (vino_use_alternative_port == enabled)
+    return;
+
+  vino_use_alternative_port = enabled;
+
+  dprintf (PREFS, "Use alternative port changed: %s\n",
+           vino_use_alternative_port ? "(true)" : "(false)");
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_use_alternative_port (l->data, vino_use_alternative_port);
+}
+
+static void
+vino_prefs_alternative_port_changed (GConfClient *client,
+                                     guint        cnxn_id,
+                                     GConfEntry  *entry)
+{
+  GSList *l;
+  int     port;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_INT)
+    return;
+
+  port = gconf_value_get_int (entry->value);
+
+  if (vino_alternative_port == port || !VINO_SERVER_VALID_PORT (port))
+    return;
+
+  vino_alternative_port = port;
+
+  dprintf (PREFS, "Alternative port changed: %d\n", vino_alternative_port);
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_alternative_port (l->data, vino_alternative_port);
+}
+
+static void
+vino_prefs_lock_screen_changed (GConfClient *client,
+			        guint	    cnxn_id,
+			        GConfEntry  *entry)
+{
+  gboolean  lock_screen;
+  GSList   *l;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  lock_screen = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (vino_lock_screen == lock_screen)
+    return;
+
+  vino_lock_screen = lock_screen;
+
+  dprintf (PREFS, "Lock Screen changed: %s\n", vino_lock_screen ? "(true)" : "(false)");
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_lock_screen (l->data, lock_screen);
+}
+
 void
 vino_prefs_create_server (GdkScreen *screen)
 {
   VinoServer *server;
 
   server = g_object_new (VINO_TYPE_SERVER,
-			 "screen",             screen,
-			 "prompt-enabled",     vino_prompt_enabled,
-			 "view-only",          vino_view_only,
-			 "auth-methods",       vino_auth_methods,
-			 "require-encryption", vino_require_encryption,
-			 "vnc-password",       vino_vnc_password,
-			 "on-hold",            !vino_enabled,
+			 "prompt-enabled",       vino_prompt_enabled,
+			 "view-only",            vino_view_only,
+			 "local-only",           vino_local_only,
+			 "use-alternative-port", vino_use_alternative_port,
+			 "alternative-port",     vino_alternative_port,
+			 "auth-methods",         vino_auth_methods,
+			 "require-encryption",   vino_require_encryption,
+			 "vnc-password",         vino_vnc_password,
+			 "on-hold",              !vino_enabled,
+			 "screen",               screen,
+                         "lock-screen",          vino_lock_screen,
 			 NULL);
 
   vino_servers = g_slist_prepend (vino_servers, server);
+  if (vino_enabled)
+    vino_mdns_start ();
 }
 
 void
@@ -284,6 +401,24 @@ vino_prefs_init (gboolean view_only)
     }
   dprintf (PREFS, "View only: %s\n", vino_view_only ? "(true)" : "(false)");
       
+  vino_local_only = gconf_client_get_bool (vino_client,
+					   VINO_PREFS_LOCAL_ONLY,
+					   NULL);
+  dprintf (PREFS, "Local only: %s\n", vino_local_only ? "(true)" : "(false)");
+
+  vino_use_alternative_port = gconf_client_get_bool (vino_client,
+                                                     VINO_PREFS_USE_ALTERNATIVE_PORT,
+                                                     NULL);
+  dprintf (PREFS, "Use alternative port: %s\n",
+           vino_use_alternative_port ? "(true)" : "(false)");
+
+  vino_alternative_port = gconf_client_get_int (vino_client,
+                                                VINO_PREFS_ALTERNATIVE_PORT,
+                                                NULL);
+  if (!VINO_SERVER_VALID_PORT (vino_alternative_port))
+    vino_alternative_port = VINO_SERVER_DEFAULT_PORT;
+  dprintf (PREFS, "Alternative port: %d\n", vino_alternative_port);
+
   vino_require_encryption = gconf_client_get_bool (vino_client,
 						   VINO_PREFS_REQUIRE_ENCRYPTION,
 						   NULL);
@@ -306,7 +441,12 @@ vino_prefs_init (gboolean view_only)
 					       VINO_PREFS_VNC_PASSWORD,
 					       NULL);
   dprintf (PREFS, "Encoded password: %s\n", vino_vnc_password ? vino_vnc_password : "(null)");
-  g_free (vino_vnc_password);
+
+  vino_lock_screen = gconf_client_get_bool (vino_client,
+                                            VINO_PREFS_LOCK_SCREEN,
+                                            NULL);
+  dprintf (PREFS, "Lock screen on disconnect: %s\n",
+           vino_lock_screen ? "(true)" : "(false)");
 
   vino_listeners [i] =
     gconf_client_notify_add (vino_client,
@@ -332,6 +472,28 @@ vino_prefs_init (gboolean view_only)
     }
   i++;
 
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_LOCAL_ONLY,
+			     (GConfClientNotifyFunc) vino_prefs_local_only_changed,
+			     NULL, NULL, NULL);
+  i++;
+
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_USE_ALTERNATIVE_PORT,
+			     (GConfClientNotifyFunc) vino_prefs_use_alternative_port_changed,
+			     NULL, NULL, NULL);
+  i++;
+
+
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_ALTERNATIVE_PORT,
+			     (GConfClientNotifyFunc) vino_prefs_alternative_port_changed,
+			     NULL, NULL, NULL);
+  i++;
+
   vino_listeners [i] =
     gconf_client_notify_add (vino_client,
 			     VINO_PREFS_REQUIRE_ENCRYPTION,
@@ -353,6 +515,14 @@ vino_prefs_init (gboolean view_only)
 			     NULL, NULL, NULL);
   i++;
 
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_LOCK_SCREEN,
+			     (GConfClientNotifyFunc) vino_prefs_lock_screen_changed,
+			     NULL, NULL, NULL);
+
+  i++;
+
   g_assert (i == VINO_N_LISTENERS);
 }
 
